using System.IO;
using System; 
using Gtk;


namespace FSpot {
	public class ChangePhotoTime : GladeDialog {
		IPhotoCollection selection;
		PhotoStore store;

		[Glade.Widget] private Gtk.Label original_time_sample;
		[Glade.Widget] private Gtk.Label new_time_sample;
		[Glade.Widget] private Gtk.SpinButton hour_spin_button;
		[Glade.Widget] private Gtk.SpinButton minute_spin_button;
		
		System.Threading.Thread command_thread;
		ThreadProgressDialog progress_dialog;
		ProgressItem progress_item;

		int photo_index;
		DateTime origTime, sampleTime;
		bool MoveCopy = false;

		public ChangePhotoTime (PhotoStore store, IPhotoCollection selection) : base ("change_time_dialog")
		{
			this.selection = selection;
			this.store = store;

			Dialog.Modal = false;
			Dialog.TransientFor = null;

			// We just need the first photo, so exit directly.
			foreach (Photo photo in selection.Photos) {
				sampleTime = photo.Time; 	// This is for the pattern sample.
				origTime = photo.Time; 		// Just to keep the original time.
				break;					
			}	

			original_time_sample.Text = origTime.ToString();			
			new_time_sample.Text = sampleTime.ToString();

			UpdateNewTimeSample();

			Dialog.ShowAll ();
			Dialog.Response += HandleResponse;
		}

		private DateTime ChangeTime (DateTime inTime )
		{
			double change_hours 	= hour_spin_button.Value;
			double change_minutes 	= minute_spin_button.Value;
			DateTime modifiedTime = inTime;
			modifiedTime = modifiedTime.AddHours 	(change_hours);
			modifiedTime = modifiedTime.AddMinutes 	(change_minutes);
			return modifiedTime;
		}


		private void UpdateNewTimeSample ()
		{
			sampleTime = ChangeTime (origTime);
			new_time_sample.Text = sampleTime.ToString();
		}

		private void on_time_spin_button_changed (object sender, System.EventArgs args)
		{	
			UpdateNewTimeSample ();
		}

		private void HandleProgressChanged (ProgressItem item)
		{
			//System.Console.WriteLine ("Changed value = {0}", item.Value);
			progress_dialog.Fraction = (photo_index - 1.0 + item.Value) / (double) selection.Photos.Length;	
		}
		

		private void DoWork() 
		{
			FSpot.ProgressItem Progress = new ProgressItem ();
			Progress.Changed += HandleProgressChanged;
			photo_index = 0;
			DateTime tmpTime;
			int NewNumberIndex = 0;
			try 
			{
				foreach (Photo photo in selection.Photos) {
					progress_dialog.Message = System.String.Format (
            	        		                         Mono.Posix.Catalog.GetString ("Changing time for picture \"{0}\""), photo.Name);
					progress_dialog.Fraction = photo_index / (double)selection.Photos.Length;
					photo_index++;
					progress_dialog.ProgressText = System.String.Format (
								Mono.Posix.Catalog.GetString ("{0} of {1}"), photo_index, selection.Photos.Length);
								
					tmpTime		= ChangeTime (photo.Time);
//System.Console.WriteLine ("Photo {0}, Old time {1}, New time {2}", photo.Name, photo.Time, tmpTime);
					photo.Time 	= tmpTime;
					store.CommitTime (photo);
	
					NewNumberIndex++; // increase the number index...
				} // foreach photo
				progress_dialog.Message = System.String.Format (
						Mono.Posix.Catalog.GetString ("Done changing time on the photos"));
				progress_dialog.Fraction = 1.0;
				progress_dialog.ProgressText = System.String.Format (
						Mono.Posix.Catalog.GetString ("Finished"));
				progress_dialog.ButtonLabel = Gtk.Stock.Ok;

			} catch (System.Exception e) {
// FIXME
// HOW to recover if failure???
			
				progress_dialog.Message = e.ToString ();
				progress_dialog.ProgressText = System.String.Format (
					Mono.Posix.Catalog.GetString ("Error changing time on photos"));
			}
		}

		
		private void HandleResponse (object sender, Gtk.ResponseArgs args)
		{
			if (args.ResponseId != Gtk.ResponseType.Ok) {
				Dialog.Destroy ();
				return;
			}
			
			command_thread = new  System.Threading.Thread (new System.Threading.ThreadStart (this.DoWork));
			command_thread.Name = Mono.Posix.Catalog.GetString ("Changing time on photos..." );
			
			Dialog.Destroy ();
			progress_dialog = new FSpot.ThreadProgressDialog (command_thread, selection.Photos.Length);
			progress_dialog.Start ();
		}
		
	} // RenameMoveCopy

} // NameSpace
